local BUSTER_TEXTURE = Resources.load_texture("widebust.png")
local SHOT_TEXTURE = Resources.load_texture("wideshot.png")
local AUDIO = Resources.load_audio("sfx.ogg")

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_SHOOT")
	action:override_animation_frames({ { 1, 2 }, { 2, 2 }, { 3, 2 }, { 4, 4 }, { 5, 1 }, { 3, 14 } })
	action:set_lockout(ActionLockout.new_animation())

	action.on_execute_func = function(self, user)
		local buster = self:create_attachment("BUSTER")
		buster:sprite():set_texture(BUSTER_TEXTURE, true)
		buster:sprite():set_layer(-1)

		local buster_anim = buster:animation()
		buster_anim:load("widebust.animation")
		buster_anim:set_state("DEFAULT")

		action:add_anim_action(5, function()
			local shot = create_wideshot(user, props)
			local tile = user:get_tile(user:facing(), 1)
			actor:field():spawn(shot, tile)
		end)
	end
	return action
end

function create_wideshot(user, props)
	local spell = Spell.new(user:team())
	spell:set_facing(user:facing())
	spell:set_hit_props(
		HitProps.new(
			props.damage,
			Hit.Impact | Hit.Flinch | Hit.Flash,
			props.element,
			user:context(),
			Drag.None
		)
	)
	local attacking = false
	local anim = spell:animation()
	spell:set_texture(SHOT_TEXTURE, true)
	spell:set_offset(0.0 * 0.5, -10 * 0.5)
	anim:load("wideshot.animation")
	anim:set_state("STARTUP")
	anim:on_complete(function()
		anim:set_state("DEFAULT")
		anim:set_playback(Playback.Loop)
		attacking = true
	end)

	spell.on_update_func = function(self)
		if not attacking then return end
		local tile = self:current_tile()
		tile:attack_entities(self)

		if self:is_sliding() == false then
			if tile:is_edge() then
				self:delete()
			end
			local dest = self:get_tile(spell:facing(), 1)
			local ref = self
			self:slide(dest, 6, function()
				if not tile:is_edge() then tile:set_state(TileState.Lava) end
			end)
		end
	end
	spell.on_collision_func = function(self, other)
		self:delete()
	end
	spell.on_delete_func = function(self)
		self:erase()
	end
	spell.can_move_to_func = function(tile)
		return true
	end

	Resources.play_audio(AUDIO)

	return spell
end
